/* (c) Winterwell 2008-2011
 * 
 */
package winterwell.utils.containers;

import java.util.Map;
import java.util.Map.Entry;

/**
 * Simple implementation of {@link java.util.Map.Entry}.
 * 
 * Supports reusing the same object via {@link #reset(Object, Object)}.
 * 
 * @author Daniel
 * 
 * @param <K>
 * @param <V>
 */
public final class MapEntry<K, V> implements Entry<K, V> {

	private K k;

	private final Map<K, V> map;
	private V v;

	public MapEntry() {
		map = null;
	}

	public MapEntry(K key, V value) {
		k = key;
		v = value;
		map = null;
	}

	public MapEntry(Map<K, V> backingMap) {
		map = backingMap;
	}

	@Override
	public K getKey() {
		return k;
	}

	@Override
	public V getValue() {
		return v;
	}

	/**
	 * Reset the key and value of this object. This allows one MapEntry object
	 * to be reused. Not sure if this is a good idea or not.
	 * 
	 * @param k
	 * @param v
	 */
	public void reset(K k, V v) {
		this.k = k;
		this.v = v;
	}

	@Override
	public V setValue(V value) {
		V old = v;
		v = value;
		// will fail if map was not set
		map.put(k, value);
		return old;
	}

	@Override
	public String toString() {
		return k + ":" + v;
	}
}
