package winterwell.utils.time;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import winterwell.utils.containers.AbstractIterator;

/**
 * Time at work
 * 
 * @author daniel
 * 
 */
public class OfficeHours implements Iterable<Period> {

	private TimeOf dayEnd;
	private TimeOf dayStart;

	Time end;
	private boolean[] onDays;
	Time start;

	/**
	 * Use this when over-riding {@link #nextOn(Time)} / {@link #nextOff(Time)}
	 */
	public OfficeHours() {

	}

	/**
	 * Default case: e.g. mon-fri, 9-5 GMT
	 * 
	 * @param days
	 *            comma separated
	 * @param dayStart
	 * @param dayEnd
	 */
	public OfficeHours(String days, TimeOf dayStart, TimeOf dayEnd) {
		onDays = new boolean[7];
		String[] daysOn = days.split(",");

		List<String> week = new ArrayList(7);
		Calendar gcal = Calendar.getInstance();
		for (int i = 0; i < 7; i++) {
			gcal.set(Calendar.DAY_OF_WEEK, i);
			String day = gcal.getDisplayName(Calendar.DAY_OF_WEEK,
					Calendar.SHORT, Locale.ENGLISH);
			day = day.substring(0, 3).toLowerCase();
			week.add(day);
		}

		for (String day : daysOn) {
			day = day.substring(0, 3).toLowerCase();
			int i = week.indexOf(day);
			onDays[i] = true;
		}

		this.dayStart = dayStart;
		this.dayEnd = dayEnd;
	}

	public Dt getTotalOfficeTime() {
		Dt total = new Dt(0);
		for (Period p : this) {
			Dt len = p.length();
			total = total.plus(len);
		}
		return total;
	}

	@Override
	public AbstractIterator<Period> iterator() {
		return new AbstractIterator<Period>() {
			Time now = start.minus(TUnit.MILLISECOND);

			@Override
			protected Period next2() throws Exception {
				Time s = nextOn(now);
				Time e = nextOff(s);
				now = e;
				return new Period(s, e);
			}
		};
	}

	public Time nextOff(Time t) {
		return nextOnOff(dayEnd, dayStart, t);
	}

	/**
	 * 
	 * @param t
	 * @return next start-on-time AFTER t (ie. != t)
	 */
	public Time nextOn(Time t) {
		return nextOnOff(dayStart, dayEnd, t);
	}

	private Time nextOnOff(TimeOf a, TimeOf b, Time t) {
		GregorianCalendar cal = t.getCalendar();
		// advance to start?
		if (a.isAfter(cal)) {
			a.advance(cal);
		}
		if (b.isBefore(cal)) {
			b.advance(cal);
		}
		// day of week?
		int day = cal.get(Calendar.DAY_OF_WEEK);
		// TODO

		return new Time(cal);
	}

}
